// These constants represent the RISC-V ELF and the image ID generated by risc0-build.
// The ELF is used for proving and the ID is used for verification.
use methods::{METHOD_ELF, METHOD_ID};

use pathfinder2::types::edge::*;
use pathfinder2::io::*;

use demo_core::Request;

use std::env;

use risc0_zkvm::{default_prover, ExecutorEnv};

fn main() {
    let args: Vec<String> = env::args().collect();
    let edges_file = &args[1];
    let edges : EdgeDB = read_edges_binary(edges_file).unwrap();
    
    
    // Initialize tracing. In order to view logs, run `RUST_LOG=info cargo run`
    env_logger::init();

    // An executor environment describes the configurations for the zkVM
    // including program inputs.
    // An default ExecutorEnv can be created like so:
    // `let env = ExecutorEnv::builder().build().unwrap();`
    // However, this `env` does not have any inputs.
    //
    // To add add guest input to the executor environment, use
    // ExecutorEnvBuilder::write().
    // To access this method, you'll need to use ExecutorEnv::builder(), which
    // creates an ExecutorEnvBuilder. When you're done adding input, call
    // ExecutorEnvBuilder::build().

    // For example:

    
    let path: Vec<Edge> = vec![Edge {
        from: "0x9BA1Bcd88E99d6E1E03252A70A63FEa83Bf1208c".into(),
        to: "0x42cEDde51198D1773590311E2A340DC06B24cB37".into(),
        token: "0x9a0bbbbd3789f184CA88f2F6A40F42406cb842AC".into(),
        capacity: "1000000000000000000".into(),
    }];
    let request: Request = Request {
        from: "0x9BA1Bcd88E99d6E1E03252A70A63FEa83Bf1208c".into(),
        to: "0x42cEDde51198D1773590311E2A340DC06B24cB37".into(),
        amount: "2343".into(),
    };

    let env = ExecutorEnv::builder()
        .write(&path)
        .unwrap()
        .write(&request)
        .unwrap()
        .write(&edges)
        .unwrap()
        .build()
        .unwrap();

    // Obtain the default prover.
    let prover = default_prover();

    // Produce a receipt by proving the specified ELF binary.
    let receipt = prover.prove_elf(env, METHOD_ELF).unwrap();

    // TODO: Implement code for retrieving receipt journal here.

    // For example:
    let _output: u32 = receipt.journal.decode().unwrap();

    // Optional: Verify receipt to confirm that recipients will also be able to
    // verify your receipt
    receipt.verify(METHOD_ID).unwrap();
}
