#![no_main]

use demo_core::{Transfer, Request};
use pathfinder2::types::Edge;
use pathfinder2::types::edge::EdgeDB;
use risc0_zkvm::guest::env;

risc0_zkvm::guest::entry!(main);

pub fn main() {
    let path: Vec<Edge> = env::read();
    
    let request: Request = env::read();
    let db: EdgeDB = env::read();
    if path.len() > 100 {
        panic!("path too long!")
    }

    println!("checking path validity...");
    let to = path.iter().fold(request.from, |acc, x| {
        if x.from == acc && x.capacity >= request.amount && db.edges().contains(&x)
        {
            x.to
        } else {
            panic!("Path invalid")
        }
    });
   assert_eq!(to, request.to, "recipient mismatch");
    //
    //    // iterator checks that
    //    // to(previous) = from(current)
    //    // from(first) = from(request)
    //    // to(last) = to(request)
    //    // capacity(from, to) < request.amount
    //
    println!("generating commitment for transfer...");
    let amount = request.amount;
    env::commit(&Transfer{path, amount}); // write public output to the journal
}
